package org.apache.bcel.generic;

import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.ElementValue;
import java.io.IOException;
import java.io.DataOutputStream;
import org.apache.bcel.classfile.ElementValuePair;

public class ElementValuePairGen
{
    private final int nameIdx;
    private final ElementValueGen value;
    private final ConstantPoolGen constantPoolGen;
    
    public ElementValuePairGen(final ElementValuePair nvp, final ConstantPoolGen cpool, final boolean copyPoolEntries) {
        this.constantPoolGen = cpool;
        if (copyPoolEntries) {
            this.nameIdx = cpool.addUtf8(nvp.getNameString());
        }
        else {
            this.nameIdx = nvp.getNameIndex();
        }
        this.value = ElementValueGen.copy(nvp.getValue(), cpool, copyPoolEntries);
    }
    
    protected ElementValuePairGen(final int idx, final ElementValueGen value, final ConstantPoolGen cpool) {
        this.nameIdx = idx;
        this.value = value;
        this.constantPoolGen = cpool;
    }
    
    public ElementValuePairGen(final String name, final ElementValueGen value, final ConstantPoolGen cpool) {
        this.nameIdx = cpool.addUtf8(name);
        this.value = value;
        this.constantPoolGen = cpool;
    }
    
    protected void dump(final DataOutputStream dos) throws IOException {
        dos.writeShort(this.nameIdx);
        this.value.dump(dos);
    }
    
    public ElementValuePair getElementNameValuePair() {
        final ElementValue immutableValue = this.value.getElementValue();
        return new ElementValuePair(this.nameIdx, immutableValue, this.constantPoolGen.getConstantPool());
    }
    
    public int getNameIndex() {
        return this.nameIdx;
    }
    
    public final String getNameString() {
        return ((ConstantUtf8)this.constantPoolGen.getConstant(this.nameIdx)).getBytes();
    }
    
    public final ElementValueGen getValue() {
        return this.value;
    }
    
    @Override
    public String toString() {
        return "ElementValuePair:[" + this.getNameString() + "=" + this.value.stringifyValue() + "]";
    }
}
